@@ -243,7 +243,7 @@ vte_bg_class_init(VteBgClass *klass)
 						  0,
 						  NULL,
 						  NULL,
-						  _vte_marshal_VOID__VOID,
+                                                  g_cclosure_marshal_VOID__VOID,
 						  G_TYPE_NONE, 0);
 	g_type_class_add_private(klass, sizeof (struct VteBgPrivate));
 }
@@ -343,16 +343,16 @@ vte_bg_cache_item_free(struct VteBgCacheItem *item)
 	/* Clean up whatever is left in the structure. */
 	if (item->source_pixbuf != NULL) {
 		g_object_remove_weak_pointer(G_OBJECT(item->source_pixbuf),
-				(gpointer*)&item->source_pixbuf);
+				(gpointer*)(void*)&item->source_pixbuf);
 	}
 	g_free(item->source_file);
 	if (item->pixmap != NULL) {
 		g_object_remove_weak_pointer(G_OBJECT(item->pixmap),
-				(gpointer*)&item->pixmap);
+				(gpointer*)(void*)&item->pixmap);
 	}
 	if (item->pixbuf != NULL) {
 		g_object_remove_weak_pointer(G_OBJECT(item->pixbuf),
-				(gpointer*)&item->pixbuf);
+				(gpointer*)(void*)&item->pixbuf);
 	}
 
 	g_slice_free(struct VteBgCacheItem, item);
@@ -441,15 +441,15 @@ vte_bg_cache_add(VteBg *bg, struct VteBgCacheItem *item)
 	bg->pvt->cache = g_list_prepend(bg->pvt->cache, item);
 	if (item->source_pixbuf != NULL) {
 		g_object_add_weak_pointer(G_OBJECT(item->source_pixbuf),
-					  (gpointer*)&item->source_pixbuf);
+					  (gpointer*)(void*)&item->source_pixbuf);
 	}
 	if (item->pixbuf != NULL) {
 		g_object_add_weak_pointer(G_OBJECT(item->pixbuf),
-					  (gpointer*)&item->pixbuf);
+					  (gpointer*)(void*)&item->pixbuf);
 	}
 	if (item->pixmap != NULL) {
 		g_object_add_weak_pointer(G_OBJECT(item->pixmap),
-					  (gpointer*)&item->pixmap);
+					  (gpointer*)(void*)&item->pixmap);
 	}
 }
 
@@ -555,7 +555,6 @@ vte_bg_get_pixmap(VteBg *bg,
 	gpointer cached;
 	GdkColormap *rcolormap;
 	GdkPixmap *pixmap;
-	GdkBitmap *mask;
 	GdkPixbuf *pixbuf;
 	char *file;
 
@@ -619,9 +618,13 @@ vte_bg_get_pixmap(VteBg *bg,
 		}
 		break;
 	case VTE_BG_SOURCE_PIXBUF:
-		pixbuf = source_pixbuf;
-		if (GDK_IS_PIXBUF(pixbuf)) {
-			g_object_ref(pixbuf);
+		if (GDK_IS_PIXBUF(source_pixbuf)) {
+                        /* If we're going to modify the pixbuf below, make a copy first! */
+                        if (saturation != 1.0) {
+                                pixbuf = gdk_pixbuf_copy(source_pixbuf);
+                        } else {
+                                pixbuf = g_object_ref(source_pixbuf);
+                        }
 		}
 		break;
 	case VTE_BG_SOURCE_FILE:
@@ -648,7 +651,6 @@ vte_bg_get_pixmap(VteBg *bg,
 	}
 
 	pixmap = NULL;
-	mask = NULL;
 	if (GDK_IS_PIXBUF(pixbuf)) {
 		/* If the image is smaller than 256x256 then tile it into a
 		 * pixbuf that is at least this large.  This is done because
@@ -657,11 +659,8 @@ vte_bg_get_pixmap(VteBg *bg,
 		pixbuf = _vte_bg_resize_pixbuf(pixbuf, 256, 256);
 		gdk_pixbuf_render_pixmap_and_mask_for_colormap(pixbuf,
 							       colormap,
-							       &pixmap, &mask,
+							       &pixmap, NULL,
 							       0);
-		if (mask != NULL) {
-			g_object_unref(mask);
-		}
 		g_object_unref(pixbuf);
 	}
 
