@@ -85,7 +85,7 @@ vte_terminal_find_charcell(VteTerminal *terminal, glong col, glong row)
 	g_assert(VTE_IS_TERMINAL(terminal));
 	screen = terminal->pvt->screen;
 	if (_vte_ring_contains(screen->row_data, row)) {
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, row);
+		rowdata = _vte_ring_index(screen->row_data, row);
 		if ((glong) rowdata->cells->len > col) {
 			ret = &g_array_index(rowdata->cells,
 					     struct vte_charcell,
@@ -117,45 +117,24 @@ vte_g_array_fill(GArray *array, gpointer item, guint final_size)
 static void
 vte_insert_line_internal(VteTerminal *terminal, glong position)
 {
-	VteRowData *row, *old_row;
-	old_row = terminal->pvt->free_row;
 	/* Pad out the line data to the insertion point. */
 	while (_vte_ring_next(terminal->pvt->screen->row_data) < position) {
-		if (old_row) {
-			row = _vte_reset_row_data (terminal, old_row, TRUE);
-		} else {
-			row = _vte_new_row_data_sized(terminal, TRUE);
-		}
-		old_row = _vte_ring_append(terminal->pvt->screen->row_data, row);
+		_vte_ring_append(terminal->pvt->screen->row_data);
 	}
 	/* If we haven't inserted a line yet, insert a new one. */
-	if (old_row) {
-		row = _vte_reset_row_data (terminal, old_row, TRUE);
-	} else {
-		row = _vte_new_row_data_sized(terminal, TRUE);
-	}
 	if (_vte_ring_next(terminal->pvt->screen->row_data) >= position) {
-		old_row = _vte_ring_insert(terminal->pvt->screen->row_data,
-				 position, row);
+		_vte_ring_insert(terminal->pvt->screen->row_data, position);
 	} else {
-		old_row =_vte_ring_append(terminal->pvt->screen->row_data, row);
+		_vte_ring_append(terminal->pvt->screen->row_data);
 	}
-	terminal->pvt->free_row = old_row;
 }
 
 /* Remove a line at an arbitrary position. */
 static void
 vte_remove_line_internal(VteTerminal *terminal, glong position)
 {
-	if (_vte_ring_next(terminal->pvt->screen->row_data) > position) {
-		if (terminal->pvt->free_row)
-			_vte_free_row_data (terminal->pvt->free_row);
-
-		terminal->pvt->free_row = _vte_ring_remove(
-				terminal->pvt->screen->row_data,
-				position,
-				FALSE);
-	}
+	if (_vte_ring_next(terminal->pvt->screen->row_data) > position)
+		_vte_ring_remove(terminal->pvt->screen->row_data, position);
 }
 
 /* Check how long a string of unichars is.  Slow version. */
@@ -183,7 +162,7 @@ vte_ucs4_to_utf8 (VteTerminal *terminal, const guchar *in)
 		outlen = (inlen * VTE_UTF8_BPC) + 1;
 
 		_vte_buffer_set_minimum_size (terminal->pvt->conv_buffer, outlen);
-		buf = bufptr = terminal->pvt->conv_buffer->bytes;
+		buf = bufptr = terminal->pvt->conv_buffer->data;
 
 		if (_vte_conv (conv, &in, &inlen, &buf, &outlen) == (size_t) -1) {
 			_vte_debug_print (VTE_DEBUG_IO,
@@ -336,27 +315,15 @@ _vte_terminal_home_cursor (VteTerminal *terminal)
 static void
 _vte_terminal_clear_screen (VteTerminal *terminal)
 {
-	VteRowData *rowdata, *old_row;
 	long i, initial, row;
 	VteScreen *screen;
 	screen = terminal->pvt->screen;
 	initial = screen->insert_delta;
 	row = screen->cursor_current.row - screen->insert_delta;
+	initial = _vte_ring_next(screen->row_data);
 	/* Add a new screen's worth of rows. */
-	old_row = terminal->pvt->free_row;
-	for (i = 0; i < terminal->row_count; i++) {
-		/* Add a new row */
-		if (i == 0) {
-			initial = _vte_ring_next(screen->row_data);
-		}
-		if (old_row) {
-			rowdata = _vte_reset_row_data (terminal, old_row, TRUE);
-		} else {
-			rowdata = _vte_new_row_data_sized(terminal, TRUE);
-		}
-		old_row = _vte_ring_append(screen->row_data, rowdata);
-	}
-	terminal->pvt->free_row = old_row;
+	for (i = 0; i < terminal->row_count; i++)
+		_vte_ring_append(screen->row_data);
 	/* Move the cursor and insertion delta to the first line in the
 	 * newly-cleared area and scroll if need be. */
 	screen->insert_delta = initial;
@@ -381,8 +348,7 @@ _vte_terminal_clear_current_line (VteTerminal *terminal)
 	 * which corresponds to the cursor. */
 	if (_vte_ring_next(screen->row_data) > screen->cursor_current.row) {
 		/* Get the data for the row which the cursor points to. */
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *,
-					  screen->cursor_current.row);
+		rowdata = _vte_ring_index(screen->row_data, screen->cursor_current.row);
 		g_assert(rowdata != NULL);
 		/* Remove it. */
 		if (rowdata->cells->len > 0) {
@@ -418,8 +384,7 @@ _vte_terminal_clear_above_current (VteTerminal *terminal)
 		if (_vte_ring_next(screen->row_data) > i) {
 			guint len;
 			/* Get the data for the row we're erasing. */
-			rowdata = _vte_ring_index(screen->row_data,
-						  VteRowData *, i);
+			rowdata = _vte_ring_index(screen->row_data, i);
 			g_assert(rowdata != NULL);
 			/* Remove it. */
 			len = rowdata->cells->len;
@@ -444,7 +409,6 @@ _vte_terminal_clear_above_current (VteTerminal *terminal)
 static void
 _vte_terminal_scroll_text (VteTerminal *terminal, int scroll_amount)
 {
-	VteRowData *row, *old_row;
 	long start, end, i;
 	VteScreen *screen;
 
@@ -458,16 +422,9 @@ _vte_terminal_scroll_text (VteTerminal *terminal, int scroll_amount)
 		end = start + terminal->row_count - 1;
 	}
 
-	old_row = terminal->pvt->free_row;
-	while (_vte_ring_next(screen->row_data) <= end) {
-		if (old_row) {
-			row = _vte_reset_row_data (terminal, old_row, FALSE);
-		} else {
-			row = _vte_new_row_data_sized(terminal, FALSE);
-		}
-		old_row = _vte_ring_append(terminal->pvt->screen->row_data, row);
-	}
-	terminal->pvt->free_row = old_row;
+	while (_vte_ring_next(screen->row_data) <= end)
+		_vte_ring_append(terminal->pvt->screen->row_data);
+
 	if (scroll_amount > 0) {
 		for (i = 0; i < scroll_amount; i++) {
 			vte_remove_line_internal(terminal, end);
@@ -1026,8 +983,7 @@ vte_sequence_handler_al (VteTerminal *terminal, GValueArray *params)
 		vte_remove_line_internal(terminal, end);
 		vte_insert_line_internal(terminal, start);
 		/* Get the data for the new row. */
-		rowdata = _vte_ring_index(screen->row_data,
-					  VteRowData *, start);
+		rowdata = _vte_ring_index(screen->row_data, start);
 		g_assert(rowdata != NULL);
 		/* Add enough cells to it so that it has the default columns. */
 		vte_g_array_fill(rowdata->cells, &screen->fill_defaults,
@@ -1147,7 +1103,7 @@ vte_sequence_handler_cd (VteTerminal *terminal, GValueArray *params)
 	i = screen->cursor_current.row;
 	if (i < _vte_ring_next(screen->row_data)) {
 		/* Get the data for the row we're clipping. */
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, i);
+		rowdata = _vte_ring_index(screen->row_data, i);
 		/* Clear everything to the right of the cursor. */
 		if ((rowdata != NULL) &&
 		    ((glong) rowdata->cells->len > screen->cursor_current.col)) {
@@ -1160,7 +1116,7 @@ vte_sequence_handler_cd (VteTerminal *terminal, GValueArray *params)
 	     i < _vte_ring_next(screen->row_data);
 	     i++) {
 		/* Get the data for the row we're removing. */
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, i);
+		rowdata = _vte_ring_index(screen->row_data, i);
 		/* Remove it. */
 		if ((rowdata != NULL) && (rowdata->cells->len > 0)) {
 			g_array_set_size(rowdata->cells, 0);
@@ -1172,19 +1128,10 @@ vte_sequence_handler_cd (VteTerminal *terminal, GValueArray *params)
 	     i++) {
 		/* Retrieve the row's data, creating it if necessary. */
 		if (_vte_ring_contains(screen->row_data, i)) {
-			rowdata = _vte_ring_index(screen->row_data,
-						  VteRowData *, i);
+			rowdata = _vte_ring_index(screen->row_data, i);
 			g_assert(rowdata != NULL);
 		} else {
-			if (terminal->pvt->free_row) {
-				rowdata = _vte_reset_row_data (terminal,
-						terminal->pvt->free_row,
-						FALSE);
-			} else {
-				rowdata = _vte_new_row_data(terminal);
-			}
-			terminal->pvt->free_row =
-				_vte_ring_append(screen->row_data, rowdata);
+			rowdata = _vte_ring_append(screen->row_data);
 		}
 		/* Pad out the row. */
 		vte_g_array_fill(rowdata->cells,
@@ -1475,9 +1422,7 @@ vte_sequence_handler_dc (VteTerminal *terminal, GValueArray *params)
 	if (_vte_ring_next(screen->row_data) > screen->cursor_current.row) {
 		long len;
 		/* Get the data for the row which the cursor points to. */
-		rowdata = _vte_ring_index(screen->row_data,
-					  VteRowData *,
-					  screen->cursor_current.row);
+		rowdata = _vte_ring_index(screen->row_data, screen->cursor_current.row);
 		g_assert(rowdata != NULL);
 		col = screen->cursor_current.col;
 		len = rowdata->cells->len;
@@ -2907,7 +2852,7 @@ vte_sequence_handler_insert_lines (VteTerminal *terminal, GValueArray *params)
 		vte_remove_line_internal(terminal, end);
 		vte_insert_line_internal(terminal, row);
 		/* Get the data for the new row. */
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, row);
+		rowdata = _vte_ring_index(screen->row_data, row);
 		g_assert(rowdata != NULL);
 		/* Add enough cells to it so that it has the default colors. */
 		vte_g_array_fill(rowdata->cells,
@@ -2956,7 +2901,7 @@ vte_sequence_handler_delete_lines (VteTerminal *terminal, GValueArray *params)
 		vte_remove_line_internal(terminal, row);
 		vte_insert_line_internal(terminal, end);
 		/* Get the data for the new row. */
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, end);
+		rowdata = _vte_ring_index(screen->row_data, end);
 		g_assert(rowdata != NULL);
 		/* Add enough cells to it so that it has the default colors. */
 		vte_g_array_fill(rowdata->cells,
@@ -3116,7 +3061,7 @@ static void
 vte_sequence_handler_screen_alignment_test (VteTerminal *terminal, GValueArray *params)
 {
 	long row;
-	VteRowData *rowdata, *old_row;
+	VteRowData *rowdata;
 	VteScreen *screen;
 	struct vte_charcell cell;
 
@@ -3126,18 +3071,10 @@ vte_sequence_handler_screen_alignment_test (VteTerminal *terminal, GValueArray *
 	     row < terminal->pvt->screen->insert_delta + terminal->row_count;
 	     row++) {
 		/* Find this row. */
-		old_row = terminal->pvt->free_row;
-		while (_vte_ring_next(screen->row_data) <= row) {
-			if (old_row) {
-				rowdata = _vte_reset_row_data (terminal, old_row, FALSE);
-			} else {
-				rowdata = _vte_new_row_data(terminal);
-			}
-			old_row = _vte_ring_append(screen->row_data, rowdata);
-		}
-		terminal->pvt->free_row = old_row;
+		while (_vte_ring_next(screen->row_data) <= row)
+			_vte_ring_append(screen->row_data);
 		_vte_terminal_adjust_adjustments(terminal);
-		rowdata = _vte_ring_index(screen->row_data, VteRowData *, row);
+		rowdata = _vte_ring_index(screen->row_data, row);
 		g_assert(rowdata != NULL);
 		/* Clear this row. */
 		if (rowdata->cells->len > 0) {
